/*
 * This class contains utility methods.
 * Most of these are print methods and useful during debugging the 
 * Tai-Daniels (TD) algorithm as
 * well as other agorithms that it uses.
 * 
 * 
 * Author: Aditya Mathur
 * Original: July 7, 2011

 * Latest update: July 24-28, 2011 Aug 11-25 2011. January 28, 20122
 * 
 */

import java.util.*;
import java.io.*;

public  class Utilities{
    
    // Debugging switches. Set to true to make debugging  active
    // false to make it inactive.
  
  static boolean graphInputDebug=false; // Debug graph input method

  static boolean sccDebug=false; // Debug SCC generation.
  static boolean graphDisplayDebug=false; // Debug SCC generation.
  
  
  
  // Useful constants for finding various types of node dependencies in the graph.
  final static int staticChain=1;
  final static int dynamicChain=2;
  final static int inheritanceChain=3;
  final static int frondEdge=4;
  
   
  final static String InheritanceEdgeLabel="eI";
  final static String AggregationEdgeLabel="eC";
  final static String AssociationEdgeLabel="eA";

  
  
  // Color constants used by the cycle decetion algorithm.
  
  final static int white=0;
  final static int grey=1;
  final static int black=2;
  
  // This variable keeps track of any possible program errors during execution.
  ArrayList<Integer> programError=new ArrayList<Integer>();
  
  //public static enum NodeColor {WHITE,GREY,BLACK};
  
  
    
    // Debug routine to display all nodes in a list.
    // Outgoing edges are displayed only when edgeAlso is true.
    
    
    static ArrayList<Node>succNodes;
    public static void displayNodes(ArrayList<Node> list, boolean edgesAlso, 
                                    boolean withWeight){
        
        if(list.size()==0){
            System.out.println("\tNo nodes to display.\n");
            return;
        }
        // List is not empty, hence display it.
        Iterator itr =list.iterator();
        while(itr.hasNext()){
            Node nextNode=(Node) itr.next();
            String abs="";
            String stub="";
           if(nextNode.isNodeAbstract())
               abs=" (Abstract) ";
           
           
            
            } // End of while  
        } // End of displayNodes()

    
    public static void displayNodeList(ArrayList<Node>n,  Node stubNode, boolean edgesAlso, boolean displayLevels) {
        if(n.size()==0){
            System.out.println("Node list is empty.");
            return;
        }
        Iterator itr=n.iterator();
        while(itr.hasNext()){
            Node next=(Node)itr.next();
            int nodeLevelMajor=next.getLevel(0);
            int nodeLevelMinor=next.getLevel(1);
            System.out.print(next.getLabel().substring(1));
            if(displayLevels)
                 System.out.print(" "+" Major:Minor level: " +nodeLevelMajor+":"+nodeLevelMinor);
                
            System.out.println();
            
           if(edgesAlso)
               displayEdges(next, false, null);
        } // End of while over nodes
        
    } // End of displayNodeList()
    
    public static String listNodeSequence(ArrayList<Node> s){
        Iterator<Node> itr =s.iterator();
        String seq="";
        while(itr.hasNext()){
            seq=seq+itr.next().getLabel().substring(1)+" ";
        }
        return seq;
    }
    // Print a graph: all nodes and the corresponding edges.
    
    public static void displayGraph(Graph g){ 
       
        Iterator itr=g.getNodes().iterator();
        int totalEdges=0;
        int nodeCount=0;
    
        while(itr.hasNext()){ 
            Node n=(Node) itr.next();
            
            nodeCount++;
            
      
            System.out.println(" Incoming/outgoing edges:"+n.getIncomingEdgeCount()+
                               "/"+n.getOutgoingEdgeCount());
            totalEdges=totalEdges+n.getOutgoingEdgeCount();
            displayEdges(n, false, null);
            System.out.println("---------");
        }// End of iteration over graph nodes.
     
        
        System.out.print("\nNodes: "+nodeCount+". Edges: "+totalEdges+ ". ");
     
       
    } // End displayGraph()
    
    
   
    // Display all edges emanating from a node. ff stubNode is not null
     // then only the edges ending at stubNode are displayed.
     
    public static void displayEdges(Node n, boolean weightDisplay, Node stubNode){
        String label=n.getLabel();
        ArrayList<Edge> edges=n.getOutgoingEdges();
        Iterator itr =edges.iterator();
        while(itr.hasNext()){
            Edge nextEdge=(Edge)itr.next();
            Node destination=nextEdge.getDestination();

            if(!(stubNode==null)){
                if(destination.equals(stubNode)){ 
                   displayEdge(nextEdge, weightDisplay);
                }
            }else{
                 displayEdge(nextEdge, weightDisplay);
            }
        }// End of while over edges.
    } // End of displayEdges()
        
 
    
    public static void displayEdges(ArrayList<Edge> e , boolean weightDisplay, Node stubNode){
        if(e.size()==0){
            System.out.println("No edges to display.");
            return;
        }
        Iterator itr =e.iterator();
        while(itr.hasNext()){
            Edge nextEdge=(Edge)itr.next();
            Node source=nextEdge.getSource();
            Node destination=nextEdge.getDestination();
            String labelSource=source.getLabel();
            String labelDestination=destination.getLabel();
            String labelEdge=nextEdge.getLabel();
           
            System.out.print("\t"+labelSource.substring(1)+"-"+"("+labelEdge.substring(1) +")"+"-"+
                             labelDestination.substring(1));
          
            System.out.println();
        }// End of edge iteration loop.
    } // End of displayEdges, another version
    
   
    public static String stringifyEdge(Edge e, boolean weightDisplay){
        String edgeString=e.getSource().getLabel().substring(1)+"-"+"("+
            e.getLabel().substring(1)+")"+"-"+e.getDestination().getLabel().substring(1);    
        
       
     
        return edgeString;
    }// End of stringifyEdge()
    
      public static String stringifyNodes(ArrayList<Node> n){
        String nodesString="";
        Iterator<Node> itr=n.iterator();
        while(itr.hasNext()){
            Node next=itr.next();
            String label=next.getLabel().substring(1);
            String root="";
            String stub="";
            String rootStub="";
            if(next.isMasterRoot()){
                root=" (MasterRoot)";
            }else{
              if(next.isRoot()){
                root=" (Root)";
              }
            }
            if(next.isStub())
                stub="(Stub)";
            rootStub=root+stub+" ";
            nodesString=label+rootStub+nodesString;
        }
        return nodesString;
    }// End of stringifyNodes()
      
      public static String stringifyNodes(Node[] n){
          String nodesString="";
          for(int i=0; i<n.length; i++){
              if(!(n[i]==null)){
                  nodesString=n[i].getLabel().substring(1)+" "+nodesString;
                  if(n[i].isStub()){
                      nodesString=nodesString+" (Stub) ";
                  }
              }
          }
          return nodesString;
      }// End of stringifyNodes()
    
    // Print all strongly connected components.
    
      
     
     
     // The following method displays one SCC. It is useful in debugging.
      public static void displayAnScc(ArrayList<Node> scc){
        
        int componentSize=scc.size();
        if(componentSize==0)
            return;
        // Loop through the ArrayList and print all SCCs.
        Iterator  itr=scc.iterator(); // This iterates over all nodes in the component
        while(itr.hasNext()){    
          Node n=(Node)itr.next();
          String frondInfo="";
         
        }// End of inner while
        System.out.println(); // Move to the next line.
      }// End of  displayAnScc for one component
      
    
      
     
     // The next method displays successors of each node in the given graph.
     
     public static void displaySuccessors(Graph g, ArrayList<String> ignoreEdges){
    
         ArrayList<Node> nodes=g.getNodes();
         System.out.println("\nAll succesors.\n");
         Iterator itr=nodes.iterator();
         while(itr.hasNext()){
             Node n=(Node)itr.next();
             ArrayList<Node> succ=n.getSuccessors(ignoreEdges);
             System.out.println("Node: "+n.getLabel());
             displayNodeList(succ,null,  true, false);
         } 
     }// End of displaySuccessors()
     
     public static void displayEdge(Edge e, boolean withWeight){
         String source=e.getSource().getLabel().substring(1);
         String destination= e.getDestination().getLabel().substring(1);
         String sccDestination=e.getSccDestination().getLabel().substring(1);
         System.out.print("\tEdge: "+source+"-"+
                          "("+e.getLabel().substring(1)+")"+"-"+
                          destination);
        
        
        
         System.out.println();
     }// End of display edge.
     

     // Method to display grammatically correct counts.
     // Edge "2 edges" or "1 edge".
     
     public static String singPlu(int n){
         
        if(n>1)
            return "s";
        else 
            return "";
     }// End of singPlu
     
     // This metod find the list of class that on hwich the inut class n depends. 
     // The method understands that there may be a cycle in the graph.
     
    
     public ArrayList<Node>  dependsOn(Node n){
         
         ArrayList<Edge> edges=n.getOutgoingEdges();
         Iterator<Edge> itr=edges.iterator();
         while(itr.hasNext()){
             Edge e=itr.next();
             Node dest=e.getDestination();
             if(dest.equals(n)){
                 succNodes.add(dest);
                 return succNodes;
             }else{
                 succNodes.add(dest);
                 dependsOn(dest);
             }  
         }
         return(succNodes);
     }// End of dependsOn()
     
   
     // Topological sort method. [Based on the CoffmanGraham algorithm.]
     // Input graph g is assumed to be a DAG.
     
     private static ArrayList<Node> topSortOrder=new ArrayList<Node>(); 
     public static ArrayList<Node> topologicalSort(Graph g){
         
       // First get nodes with no incoming edges.
       // Make sure that deleted edges are ignored.
         ArrayList<Node>noIncomingEdges=g.getNodesNoIncomingEdgesIgnoreDeleted();
        
         Iterator<Node> itr=noIncomingEdges.iterator();
         
         g.resetMarks(); // Set all nodes to not visited.
         while(itr.hasNext()){
             Node n=itr.next(); 
             visit(n);
         }
         return topSortOrder;
     }// End of topSort()
     
     
     
     
     private static void visit(Node n){
       if (!n.getVisitStatus()){
         n.setVisitStatus(true);
         
         ArrayList<Edge> edges=n.getOutgoingEdges();
         
         Iterator<Edge> itr=edges.iterator();
         while(itr.hasNext()){
           Edge e=itr.next(); 
           if(!e.isDeleted()){
             
             Node m;
             m= e.getSccDestination();
             
             visit(m); 
          }
         }// End of edge iteration
         topSortOrder.add(n);
        
       }// End of if
     }// End of visit()
     
     
     // Display test sequence recursively. Used for debugging.
     public static void leTraonTestSequence(ArrayList<ArrayList<Node>> scc){
       System.out.println("Current test sequence:");
       
       Iterator<ArrayList<Node>> itr=scc.iterator();
       int serial=0;
       while(itr.hasNext()){
         ArrayList<Node> anScc=itr.next();
         serial++;
         String nodeList=stringifyNodes(anScc);
         System.out.println(serial+". "+"( "+nodeList+")");
       }
       
     }// End of laTraonTestSequence()
     
    
    
     
     // Reverse the node list so it
     // it is in the order in which nodes are to be tested.
     
     private static void reverseNodeList(ArrayList<Node> n){
         int front=0;
         int tail=n.size()-1;
         while(front<tail){
             Node f=n.get(front); // Save reference to front node.
             n.set(front, n.get(tail)); // Replace this poisition by the tail node
             n.set(tail, f); // Please frot node at tail;
             front++; // Move front one step ahead and
             tail--;   // tail one step backwards.
         }

     }// End of reverseNodeList()
     
       public static void debug(boolean debugFlag, String message){
         if(debugFlag)
             System.out.println(message);
     }// End of debug()
       
       
       
       
       // Colored DFS algorithm for detecting whether or not
       // a graph (collection of nodes/edges), has a cycle.
       
       // Reference: www.cs.berkeley.edu/~kamil/teaching/sp03/041403.pdf
       
      
       public static boolean isThereACycle(ArrayList<Node> nodes){
         Iterator<Node> itr=nodes.iterator();
         while(itr.hasNext()){
           Node n=itr.next();
           n.setColor(white);
           n.setVisitStatus(false);
         
         }// End of loop to mark nodes as not visited.
         
         itr=nodes.iterator();
         while(itr.hasNext()){
           Node n=itr.next();
           if (n.getColor()==white) {
             if (visitNode(nodes, n)){
               return true;
             }
           }
         }// End of second node iterator
         return false;
       }// End of isThereACycle()
       
       private static boolean visitNode(ArrayList<Node> nodes, Node v){
           v.setColor(grey);
           ArrayList<Edge> edges=v.getOutgoingEdges(nodes);
           Iterator<Edge> itr=edges.iterator();
           while(itr.hasNext()){
               Edge e=itr.next();
               Node u=e.getDestination();
               if (u.getColor() == grey){ 
                   return true;
               }
               else {
                   if (u.getColor() == white){
                       if (visitNode(nodes, u)) {
                           return true;
                       }
                   }
               }// End of else
           }// End of edge iterator
           v.setColor(black);
           return false;
       }// End of visitNode()
       
       
       // This is to test methods in this class.
       public static void main(String [] args){
         
       }
           
     
       
} // End of class Utilities