/*
 * Class Edge is used in the algorithm for generating
 * sequence of classes in integration testing
 * using various algorithms for class order determination. 
 * It is used in class Node to contain edges going out of a node.
 * 
 * Original: July 7, 2011.
 * Latest update: Aug 11, 2011
 * 
 * Added masterRootDestinaion to assist with tst order determination in YLT et al. algorithm.
 * Aditya Mathur
 * 
 * Note:
 * 
 * I had forgotten to initialize  masterRootDestination in the constructor. This
 * did not create any problem with the YLT et al. example. But
 * it caused a null pointer error during test order determination
 * when applied to the simple example in  Braind et al.'s paper.
 * */

public class Edge{
    
    Node source; // Source node of the edge (this is redundant though at times useful).
    Node destination; // Destination node of edge (e.g., node Y in edge  X--Y).
    Node masterRootDestination; // THis is used during test order determination.
    String label; // Edge label. 
    boolean visited; // Used in graph traversal, e.g., Depth First Search.
    boolean frond; // Used in LeTraon et al. and Briand et al. algorithms.
    boolean deleted; // used in Le Traon algorithm
    EdgeWeight w; // Weight of an edge
    public static String deleteString="*"; // Used to mark an edge as deleted.
    
    // For the Tai-Daniels algorithm an edge label can be one of the following:
    // eA: Association. eC: Collection (Aggregation) and eI  Inheritance. The prefix e indicates
    // that the label is that of an edge. A prefix of n indicates a node.
    
    // An edge is always associated with a node, its source node.
    //  Hence only a destination node is needed to define an edge. Nevertheless,
    // to make things a bit simple, we also add a source node to each edge.
    
    // The weight of an edge e is =s+d, where s is the number of incoming edges to the
    // source node of e and d is the number of edges outgoing from
    // the destination node of e. [Tai-Daniels refer to source and destination
    // as, respectively, head and tail.]
    
    
       
      // Create an empty edge.
    public Edge(){ 
        source=null;
        destination=null;
        label="Undefined";  
        visited=false;
        frond=false;
        deleted=false;
        w=new EdgeWeight(0, 0, 0);
        frond=false;
    }
    // Create a non-trivial edge with start and destination nodes as s and d, respectively.
    public Edge(Node s, Node d, String l){ 
        source=s;
        destination=d;
        masterRootDestination=d; 
        label=l;   
        visited=false;
        frond=false;
        deleted=false;
        w=new EdgeWeight(0, 0, 0);
    }
    
    public Node getSource(){ // Extract source node.
        return source;
    }
    
    public Node getDestination(){ // Extract destination node.
        return destination;
    }
    
      public Node getMasterRootDestination(){
          return masterRootDestination;
          
      }
    
     public EdgeWeight getEdgeWeight(){
        return w;
    }
    public int getWeight(){
        return w.getWeight();
    }
    
     public int getWeightIn(){
        return w.getWeightIn();
    }
     
      public int getWeightOut(){
       return w.getWeightOut();
    }
      
    
      public boolean isDeleted(){
          return deleted;
      }
      
      public boolean isFrond(){
          return frond;
      }
    
    public void setWeight(EdgeWeight w){
       this.w=w; 
    }
    
     public void setEdgeWeight(EdgeWeight w){
       this.w=w; 
    }

     public void markFrond(){
         frond=true;
     }
     
    
    // This method is used when the label
    // of an edge is changed in the Tai-Danils algorithm
    // to indicate that it has been deleted from the graph.
    // Thus, the edge remains in the graph except that an asterisk is appended to its label.
    // For example, a label "eA" would become "eA*"
    
    public void delete(){
      if(!deleted){
        this.label=label+deleteString;
        deleted=true;
        destination.edgeDeleted();
        }
    }
    
    public void setDestination(Node n){
        destination=n;
    }
    
      public void setMasterRootDestination(Node n){
        masterRootDestination=n;
    }
    public String getLabel(){ // Extract label.
        return label;
    }
    
    public void markVisited(){
        visited=true;
    }
    
     public void markNotVisited(){
        visited=false;
    }
     
     public boolean getVisited(){
         return visited;
     }
}