/*
 * Class Graph used in the algorithm for generating
 * sequence of classes in integration testing
 * using Tai-Daniels algorithm.
 * 
 * July 7, 2011.
 * Aditya Mathur
 * */

import java.util.*;
public class Graph{
    
    ArrayList <Node> nodes;
    int nodeCount;
  
    // Construct an empty graph.
    public Graph(){ 
        nodes=new ArrayList<Node>();
        nodeCount=0;
    
    }// End Graph()
    
    // Construct a graph with given nodes.
    
    public Graph(ArrayList<Node> n){ 
        nodes=n;
        nodeCount=nodes.size();
        resetMarks(); // Mark all nodes to indicate not visited.
    }// End Graph(n)
    
    // Add a node to the graph but only if
    // a node with the same label does not exist.
    public void addNode(Node n){ 
       // If the graph is empty then just ad this node.
        if(nodeCount==0){
            nodes.add(n);
            nodeCount++;
            return;
        }
        
        // Otherwise add only if a node with the same label does not exist.
        String label1=n.getLabel();
        Iterator itr =nodes.iterator();
        while(itr.hasNext()){
            Node next=(Node)itr.next();
            String label2=next.getLabel();
            if(label1.equals(label2)){
                System.out.println("Node with the same label detected and ignored.");
                return;
            } 
        }// End of while
        
        // Node does not exist in the graph, hence add.
        nodes.add(n);
        nodeCount++;
    }
    
    public Node removeNode(int index){
        return (nodes.remove(index));
    }
    
    public Node getNode(int index){ // REturn node at a given index.
        return (nodes.get(index));
    }

    
    public ArrayList<Node> getNodes(){ // Return all nodes in the graph.
      return nodes;
    } // End of getNodes()
    
    // The following method returns nodes at  major level m.
    public ArrayList<Node> getNodes(int m){ // Return nodes at level m.
        Iterator itr=nodes.iterator();
        ArrayList<Node> ML=new ArrayList<Node>(); // This is where we keep the nodes to be retruned.
        while(itr.hasNext()){
            Node n=(Node)itr.next();
            if(n.getLevel(0)==m){
                ML.add(n);
            }
        }
        return(ML);
    }// End of getNodes()
    
     // The following method returns nodes at  major level m and minor level l.
    
    public ArrayList<Node> getNodes(int m, int l){ // Return nodes at level l.
        Iterator itr=nodes.iterator();
        ArrayList<Node> ML=new ArrayList<Node>(); // This is where we keep the nodes to be retruned.
        while(itr.hasNext()){
            Node n=(Node)itr.next();
            if(n.getLevel(0)==m && n.getLevel(1)==l){
                ML.add(n);
            }
        }
        return(ML);
    }// End of getNodes()
    
    
    // Reset all nodes to not visited.
    public void resetMarks(){ 
       
        Iterator itr=nodes.iterator();
        while(itr.hasNext()){ 
            Node n=(Node) itr.next();
            n.markNotVisited(); // This and the next are for Tarjan's algorithm.
            n.setIndex(-1); 
        } 
    } // End of resetMarks()
    
    public void resetNodeAndEdgeMarks(ArrayList<Node> nodes){
        Iterator <Node> itr=nodes.iterator();
        while(itr.hasNext()){
            Node n=itr.next();
            n.markNotVisited();
            n.setIndex(-1); 
        }// End of while
    }// End of  resetNodeMarks
    
}